#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Price channel regression
    /// </summary>
    [Description("Price channel regression")]
    public class DeanSemaphore : Indicator
    {
        #region Variables
        // Wizard generated variables
            private double stdDevWidth = 2; // Default setting for StdDevWidth
            private int stdDevPeriod = 60; // Default setting for StdDevPeriod
            private int wcciPeriod = 25; // Default setting for WcciPeriod
            private int wcciPeriodTurbo = 6; // Default setting for WcciPeriodTurbo
        // User defined variables (add any user defined variables below)
		private int varBlank ;
		private int varR2Buy ;
		private int varR2Sell;
		private int varGBuy;
		private int varGSell;
		
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
           
			
			Add(new Plot(new Pen(Color.Yellow, 5), PlotStyle.Bar, "Ready2Buy"));
			Add(new Plot(new Pen(Color.Yellow, 5), PlotStyle.Bar, "Ready2Sell"));
			Add(new Plot(new Pen(Color.Green, 5), PlotStyle.Bar, "GoBuy"));
			Add(new Plot(new Pen(Color.Red, 5), PlotStyle.Bar, "GoSell"));

			
            Add(new Plot(Color.FromKnownColor(KnownColor.White), PlotStyle.Bar, "Blank"));
			Add(new Line(Color.DarkGray, 0, "Zero line"));			
			
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			
			if (CurrentBar < 20) return;
		
			varBlank = 0;
			//varR2Sell = 0;
			//varR2Buy = 0;
			varGBuy = 0;
			varGSell = 0;
			
			if(RegressionChannel(stdDevPeriod,stdDevWidth).Lower[0]>= RHeikenAshi().HALow[0])
			{
				varR2Buy = 1;
				varBlank = 1;
				varR2Sell = 0;
				varGBuy = 0;
				varGSell = 0;
				
				
			}
			if(RegressionChannel(stdDevPeriod,stdDevWidth).Upper[0]<= RHeikenAshi().HAHigh[0])
			{
				varR2Sell = -1;
				varBlank = -1;
				varR2Buy = 0;
				varGBuy = 0;
				varGSell = 0;
								
			}
			
			if(varR2Buy == 1 
				&& RHeikenAshi().HAOpen[0] < RHeikenAshi().HAClose[0]
				&&  Close[0] > LinReg(Close,wcciPeriod )[0])
			{
				varGBuy = 2;
				varBlank = 2;
				varR2Buy = 0;
				varR2Sell = 0;
				varGSell = 0;
				
				if(RegressionChannel(stdDevPeriod,stdDevWidth).Lower[0]>= RegressionChannel(stdDevPeriod,stdDevWidth).Lower[5])
				{
					varBlank = 3;
					Alert("L", NinjaTrader.Cbi.Priority.High, "Go Long", "Alert2.wav", 1, Color.Green, Color.White);
				}
	
			}
			if(varR2Sell == -1 
				&& RHeikenAshi().HAOpen[0]  > RHeikenAshi().HAClose[0]
				&& Close[0] < LinReg(Close,wcciPeriod )[0])
			{
				varGSell = -2;
				varBlank = -2;
				varR2Sell = 0;
				varR2Buy = 0;
				varGBuy = 0;
				
				if(RegressionChannel(stdDevPeriod,stdDevWidth).Lower[0]<= RegressionChannel(stdDevPeriod,stdDevWidth).Lower[5])
				{
					varBlank = -3;
					Alert("S", NinjaTrader.Cbi.Priority.High, "Go Short", "Alert2.wav", 1, Color.Red, Color.White);
				}				
								
			}
			
			Blank.Set(varBlank);
			Ready2Buy.Set(varR2Buy);
			Ready2Sell.Set(varR2Sell);
			GoBuy.Set(varGBuy);
			GoSell.Set(varGSell);
					
			
           
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Ready2Buy
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Ready2Sell
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries GoBuy
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries GoSell
        {
            get { return Values[3]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Blank
        {
            get { return Values[4]; }
        }
				
		
		
		
		
		
		
		
		
		
		

        [Description("How many deviations")]
        [Category("Parameters")]
        public double StdDevWidth
        {
            get { return stdDevWidth; }
            set { stdDevWidth = Math.Max(0.001, value); }
        }

        [Description("How many bars back")]
        [Category("Parameters")]
        public int StdDevPeriod
        {
            get { return stdDevPeriod; }
            set { stdDevPeriod = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int WcciPeriod
        {
            get { return wcciPeriod; }
            set { wcciPeriod = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int WcciPeriodTurbo
        {
            get { return wcciPeriodTurbo; }
            set { wcciPeriodTurbo = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DeanSemaphore[] cacheDeanSemaphore = null;

        private static DeanSemaphore checkDeanSemaphore = new DeanSemaphore();

        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        public DeanSemaphore DeanSemaphore(int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            return DeanSemaphore(Input, stdDevPeriod, stdDevWidth, wcciPeriod, wcciPeriodTurbo);
        }

        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        public DeanSemaphore DeanSemaphore(Data.IDataSeries input, int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            checkDeanSemaphore.StdDevPeriod = stdDevPeriod;
            stdDevPeriod = checkDeanSemaphore.StdDevPeriod;
            checkDeanSemaphore.StdDevWidth = stdDevWidth;
            stdDevWidth = checkDeanSemaphore.StdDevWidth;
            checkDeanSemaphore.WcciPeriod = wcciPeriod;
            wcciPeriod = checkDeanSemaphore.WcciPeriod;
            checkDeanSemaphore.WcciPeriodTurbo = wcciPeriodTurbo;
            wcciPeriodTurbo = checkDeanSemaphore.WcciPeriodTurbo;

            if (cacheDeanSemaphore != null)
                for (int idx = 0; idx < cacheDeanSemaphore.Length; idx++)
                    if (cacheDeanSemaphore[idx].StdDevPeriod == stdDevPeriod && Math.Abs(cacheDeanSemaphore[idx].StdDevWidth - stdDevWidth) <= double.Epsilon && cacheDeanSemaphore[idx].WcciPeriod == wcciPeriod && cacheDeanSemaphore[idx].WcciPeriodTurbo == wcciPeriodTurbo && cacheDeanSemaphore[idx].EqualsInput(input))
                        return cacheDeanSemaphore[idx];

            DeanSemaphore indicator = new DeanSemaphore();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.StdDevPeriod = stdDevPeriod;
            indicator.StdDevWidth = stdDevWidth;
            indicator.WcciPeriod = wcciPeriod;
            indicator.WcciPeriodTurbo = wcciPeriodTurbo;
            indicator.SetUp();

            DeanSemaphore[] tmp = new DeanSemaphore[cacheDeanSemaphore == null ? 1 : cacheDeanSemaphore.Length + 1];
            if (cacheDeanSemaphore != null)
                cacheDeanSemaphore.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDeanSemaphore = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DeanSemaphore DeanSemaphore(int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            return _indicator.DeanSemaphore(Input, stdDevPeriod, stdDevWidth, wcciPeriod, wcciPeriodTurbo);
        }

        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        public Indicator.DeanSemaphore DeanSemaphore(Data.IDataSeries input, int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            return _indicator.DeanSemaphore(input, stdDevPeriod, stdDevWidth, wcciPeriod, wcciPeriodTurbo);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DeanSemaphore DeanSemaphore(int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            return _indicator.DeanSemaphore(Input, stdDevPeriod, stdDevWidth, wcciPeriod, wcciPeriodTurbo);
        }

        /// <summary>
        /// Price channel regression
        /// </summary>
        /// <returns></returns>
        public Indicator.DeanSemaphore DeanSemaphore(Data.IDataSeries input, int stdDevPeriod, double stdDevWidth, int wcciPeriod, int wcciPeriodTurbo)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DeanSemaphore(input, stdDevPeriod, stdDevWidth, wcciPeriod, wcciPeriodTurbo);
        }

    }
}
#endregion
