
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Heiken-Ashi redesigned by roonius")]
	public class RHeikenAshi : Indicator
    {
        #region Variables
		private Color	barColorUp 		= Color.Lime;
		private Color	barColorDown 	= Color.Red;
		private Color	shadowColor 	= Color.Black;
		private int		shadowWidth		= 1;
		private Color	saveDownColor	= Color.Empty;
		private Pen		savePen			= null;
		private Color	saveUpColor		= Color.Empty;
		private SolidBrush brushUp = null;
		private	SolidBrush brushDown = null;
		private Pen myPen = null;
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(Color.Transparent, PlotStyle.Line, "HAOpen"));
			Add(new Plot(Color.Transparent, PlotStyle.Line, "HAHigh"));
			Add(new Plot(Color.Transparent, PlotStyle.Line, "HALow"));
			Add(new Plot(Color.Transparent, PlotStyle.Line, "HAClose"));
			brushUp = new SolidBrush(barColorUp);
			brushDown = new SolidBrush(barColorDown);
			myPen = new Pen(shadowColor,shadowWidth);
			PlotsConfigurable 	= false;
			PaintPriceMarkers 	= false;
			Overlay				= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			if (ChartControl != null && saveDownColor == Color.Empty && ChartControl.ChartStyle.DownColor != Color.Transparent)
			{
				saveDownColor	= ChartControl.ChartStyle.DownColor;
				saveUpColor		= ChartControl.ChartStyle.UpColor;
				savePen			= ChartControl.ChartStyle.Pen;

				ChartControl.ChartStyle.DownColor	= Color.Transparent; 
				ChartControl.ChartStyle.UpColor		= Color.Transparent; 
				ChartControl.ChartStyle.Pen			= new Pen(Color.Transparent); 
			}
			if (CurrentBar == 0)
			{
				Values[0].Set(0);
				Values[1].Set(0);
				Values[2].Set(0);
				Values[3].Set(0);
				return;
			}
			
			Values[3].Set((Open[0] + High[0] + Low[0] + Close[0]) / 4); // Calculate the close
			Values[0].Set((Values[0][1] + Values[3][1]) / 2); // Calculate the open
			Values[1].Set(Math.Max(High[0], Values[0][0])); // Calculate the high
			Values[2].Set(Math.Min(Low[0], Values[0][0])); // Calculate the low

		}
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
//			 base.Plot(graphics, bounds, min, max);
            int barPaintWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			int bars = ChartControl.BarsPainted;
			int barcount = 0;
			while (barcount<ChartControl.BarsPainted)
			{
						int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + barcount;
						if (idx < 0 || idx >= Input.Count  || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
						{ barcount++; continue;}
						double valH	= HAHigh.Get(idx);				
						double valL	= HALow.Get(idx);			
						double valC	= HAClose.Get(idx);			
						double valO	= HAOpen.Get(idx);	
						int center = (barPaintWidth%2==0)?1:0;
						int	x = (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barPaintWidth / 2 + (barcount - ChartControl.BarsPainted + 1) * ChartControl.BarSpace);
						int	y1 = (int) ((bounds.Y + bounds.Height) - ((valO - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
						int	y2 = (int) ((bounds.Y + bounds.Height) - ((valH - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
						int	y3 = (int) ((bounds.Y + bounds.Height) - ((valL - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
						int	y4 = (int) ((bounds.Y + bounds.Height) - ((valC - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);
						if (y4==y1) graphics.DrawLine(myPen,x-(int)(barPaintWidth/2)+1, y1, x-barPaintWidth/2+barPaintWidth-2-center, y1);
						else
						{
							graphics.DrawLine(myPen, x, y2, x, y3);
							if (y4>y1) 
							{
								graphics.FillRectangle(brushDown,x-(int)(barPaintWidth/2)+1,y1,barPaintWidth-2-center,y4-y1);
							} 
							if (y4<y1)
							{
								graphics.FillRectangle(brushUp,x-(int)(barPaintWidth/2)+1,y4,barPaintWidth-2-center,y1-y4);
							}
						}
					barcount++;
			} 
		}
		
		

        #region Properties
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of down bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Down color")]
		public Color BarColorDown
		{
			get { return barColorDown; }
			set { barColorDown = value; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string BarColorDownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorDown); }
			set { barColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of up bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Up color")]
		public Color BarColorUp
		{
			get { return barColorUp; }
			set { barColorUp = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string BarColorUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(barColorUp); }
			set { barColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of shadow line.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Shadow color")]
		public Color ShadowColor
		{
			get { return shadowColor; }
			set { shadowColor = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string ShadowColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(shadowColor); }
			set { shadowColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Width of shadow line.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Shadow width")]
		public int ShadowWidth
		{
			get { return shadowWidth; }
			set { shadowWidth = Math.Max(value, 1); }
		}
		
		/// <summary>
		/// Gets the RHeikenAshi Open value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAOpen
		{
			get { return Values[0]; }
		}
				
		/// <summary>
		/// Gets the RHeikenAshi High value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAHigh
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// Gets the RHeikenAshi Low value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HALow
		{
			get { return Values[2]; }
		}

		/// <summary>
		/// Gets the RHeikenAshi Close value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries HAClose
		{
			get { return Values[3]; }
		}
        #endregion

        #region Miscellaneous
        public override void Dispose()
        {
            if (ChartControl != null && saveDownColor != Color.Empty && ChartControl.ChartStyle.DownColor == Color.Transparent)
            {
				ChartControl.ChartStyle.DownColor	= saveDownColor;
				ChartControl.ChartStyle.UpColor		= saveUpColor;
				ChartControl.ChartStyle.Pen			= savePen;
            }
            base.Dispose();
        }
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RHeikenAshi[] cacheRHeikenAshi = null;

        private static RHeikenAshi checkRHeikenAshi = new RHeikenAshi();

        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        public RHeikenAshi RHeikenAshi()
        {
            return RHeikenAshi(Input);
        }

        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        public RHeikenAshi RHeikenAshi(Data.IDataSeries input)
        {

            if (cacheRHeikenAshi != null)
                for (int idx = 0; idx < cacheRHeikenAshi.Length; idx++)
                    if (cacheRHeikenAshi[idx].EqualsInput(input))
                        return cacheRHeikenAshi[idx];

            RHeikenAshi indicator = new RHeikenAshi();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            RHeikenAshi[] tmp = new RHeikenAshi[cacheRHeikenAshi == null ? 1 : cacheRHeikenAshi.Length + 1];
            if (cacheRHeikenAshi != null)
                cacheRHeikenAshi.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRHeikenAshi = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RHeikenAshi RHeikenAshi()
        {
            return _indicator.RHeikenAshi(Input);
        }

        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        public Indicator.RHeikenAshi RHeikenAshi(Data.IDataSeries input)
        {
            return _indicator.RHeikenAshi(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RHeikenAshi RHeikenAshi()
        {
            return _indicator.RHeikenAshi(Input);
        }

        /// <summary>
        /// Heiken-Ashi redesigned by roonius
        /// </summary>
        /// <returns></returns>
        public Indicator.RHeikenAshi RHeikenAshi(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RHeikenAshi(input);
        }

    }
}
#endregion
